#!/usr/bin/perl

=begin

	(filename)

	reform.plx

	(copyright)

	Copyright (c) 2013 Alexej Magura

	This file is part of Reform.

	Reform is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Reform is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Reform.  If not, see <http://www.gnu.org/licenses/>.

=cut

use File::Basename qw(basename);
use File::Temp;
use Getopt::Long;
use Text::Diff qw(diff);

Getopt::Long::Configure("bundling");

use strict; 
use warnings;

# Getopt Switch-Flags 

my $help=0;
my $version=0;
my $verbose=0;

# Getopt Value Vars

my $version_string='0.1.1';

GetOptions ("help|h" => \$help,
	    "version" => \$version,
	    "verbose|v" => \$verbose)
    or exit 2;



if ( $help) {

    printf ( "usage: %s ", basename($0));
    print  ( "[option]... <file>... \n",
	     "\n",
	     "  -h, --help\t\tprint this message and exit\n",
	     "  -v, --verbose\t\tprint (unified) diff of the before and after file(s)\n",
	     "      --version\t\tprint version and exit\n\n");

	 exit 0;

}
elsif ( $version) {

    printf ( "%s\n", $version_string);
    exit 0;
}




sub main {


    my $PREPARE=0;
    my $BUILD=0;
    my $PACKAGE=0;
    my $fh = File::Temp->new( SUFFIX => '.tmp', TEMPLATE => 'reform-XXXXXXX', DIR => '/tmp', UNLINK => 0); 
    my $fname = $fh->filename;

    if ( not ( @_)) {

	exit 3;

    }

    foreach my $file (@_) {

	if ( not (-f $file)) {

	    exit 1;
	}

	open ( FILE_a, "<$file");
	open ( FILE_b, ">$fname");


	while (<FILE_a>) {

	    if ( $_ =~ /^\s*build\s*\(\)/ ) {

		$BUILD=1; 

	    }

	    if ( $_ =~ /^\s*package\s*\(\)/ ) {

		$PACKAGE=1;

	    }

	    if ( $_ =~ /^\s*prepare\s*\(\)/ ) {

		$PREPARE=1;

	    }

	}

	seek(FILE_a, 0, 0);

	while (<FILE_a>) {

	    if ( $BUILD and (not $PACKAGE)) {

		$_ =~ s/^(\s*)build(\s*)\(\)/$1package$2\(\)/;

	    }
	    
	    $_ =~ s/\$startdir\/src(\/?)/\$srcdir$1/g;
	    $_ =~ s/\$startdir\/pkg(\/?)/\$pkgdir$1/g;


	    print FILE_b "$_";

	}

	close(FILE_a);
	close(FILE_b);

	if ( $verbose) {

	    print diff $file => $fname;


	}

	open(FILE_a, ">$file");
	open(FILE_b, "<$fname");

	while (<FILE_b>) {

	    print FILE_a "$_";
	
	}

	close(FILE_a);
	close(FILE_b);

	unlink $fname;

    }
}

main(@ARGV);
